/*
 * tek-getwave: get waveforms from Tektronix MDO series oscilloscopes
 * Copyright © 2020 D Scott Guthridge <scott_guthridge@rompromity.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A11 PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "archdep.h"

#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>

#ifndef HAVE_STRDUP

/*
 * _tek_getwave_strdup: duplicate a string
 *	(XOPEN >= 500, POSIX >= 200809, BSD, SVID)
 */
char *_tek_getwave_strdup(const char *s)
{
    size_t length = strlen(s);
    char *buf;

    if ((buf = malloc(length + 1)) == NULL) {
	return NULL;
    }
    (void)strcpy(buf, s);
    return buf;
}
#endif /* HAVE_STRDUP */
#ifndef HAVE_VASPRINTF

/*
 * _tek_getwave_vasprintf: print to allocated string (GNU)
 */
int _tek_getwave_vasprintf(char **strp, const char *fmt, va_list ap)
{
    char *buf;
    int rv;

    if (strp == NULL) {
	errno = EINVAL;
	return -1;
    }
#define MAX	4095
    if ((buf = malloc(MAX + 1)) == NULL) {
	return -1;
    }
    (void)memset((void *)buf, '\000', MAX + 1);
    if ((rv = vsnprintf(buf, MAX + 1, fmt, ap)) == -1) {
	free((void *)buf);
	return -1;
    }
    if (rv < (MAX + 1) / 2) {
	char *cp;
	
	if ((cp = realloc(buf, rv + 1)) != NULL) {
	    buf = cp;
	}
    }
    *strp = buf;
    return rv;
}
#endif /* HAVE_VASPRINTF */
